import Foundation

struct ThumbnailListGenerator {

    private let factory: TemplateFactory

    init(factory: TemplateFactory) {
        self.factory = factory
    }

    func generateContent(items: [Element], parent: Element, language: String, style: ThumbnailStyle) -> String {
        items.map { itemContent($0, parent: parent, language: language, style: style) }
            .joined(separator: "\n")
    }

    private func itemContent(_ item: Element, parent: Element, language: String, style: ThumbnailStyle) -> String {

        let metadata = item.localized(for: language)
        var content = [ThumbnailKey : String]()

        if item.state.hasThumbnailLink {
            let fullPageUrl = item.fullPageUrl(for: language)
            let relativePageUrl = parent.relativePathToFileWithPath(fullPageUrl)
            content[.url] = "href=\"\(relativePageUrl)\""
        }

        let (thumbnailSourcePath, thumbnailDestPath) = item.thumbnailFilePath(for: language)
        let thumbnailDestNoExtension = thumbnailDestPath.dropAfterLast(".")
        content[.image] = parent.relativePathToFileWithPath(thumbnailDestNoExtension)

        if style == .large, let suffix = metadata.thumbnailSuffix {
            content[.title] = factory.html.make(title: metadata.title, suffix: suffix)
        } else {
            content[.title] = metadata.title
        }
        content[.corner] = item.cornerText(for: language).unwrapped {
            factory.largeThumbnail.makeCorner(text: $0)
        }

        files.requireMultiVersionImage(
            source: thumbnailSourcePath,
            destination: thumbnailDestNoExtension + ".jpg",
            requiredBy: item.path,
            width: style.width,
            desiredHeight: style.height)

        return factory.thumbnail(style: style).generate(content, shouldIndent: false)
    }
}
